/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
#define _HEAVY_CONTEXT_WSTD_SMTHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlVar.h"
#include "HvSignalVar.h"
#include "HvSignalEnvelope.h"
#include "HvControlCast.h"
#include "HvControlBinop.h"
#include "HvSignalLine.h"
#include "HvControlIf.h"
#include "HvMath.h"

class Heavy_WSTD_SMTHR : public HeavyContext {

 public:
  Heavy_WSTD_SMTHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_SMTHR();

  const char *getName() override { return "WSTD_SMTHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
        SMTHR = 0xD88E26F9, // Smthr
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_mQpFYpPQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iePXs7ST_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_qietrNeU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZLjKZKJI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_3p8Do31j_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_imQbn5LQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_fRuXJiqG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_pX8Os2Vp_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_9B9V5TpW_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Fevrto5j_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_JOumrFhE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_nKJ7Qc8o_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_u9UMbfJZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_qnRWmulD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_fa1OhshP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_oQJfRBRd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Ky3QqWqh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_o4iLABqI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_neiSfUkE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_SWja4r7G_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_uBYKen0V_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_oCCTvbyz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Ov6Hs5mB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_IQPIqnHm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_yzJFrBx0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_DtwdFIjZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_V7v10piL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_RHCeAfcI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_n4Qlxgdj_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_jsRAYtwZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_aYWkMz6L_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_wTd7zwY0_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_AGATjo6q_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_Cs3tmMiG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_pm5YmjXV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_1Qu7FPO6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_aPtwgfDv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_XkoRtimk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_7CqoGX8H_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_VFgZglKk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Choz4NEz_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_5U27tc7A_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_bvfFQz3I_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_4PxWXink_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1VR10VXc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_I1S2yldu_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_vjdZUcB6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_1Zqtvxnl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_8WN0aAH5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_iNp1IwRl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_gUIsZvSm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_Ow3lyY4t_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_Zw9HbTCN_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_B9f1D9GQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_BSxvRsgs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_L4EO7Nsd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_HJWMH9Jg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Zoi9Dxlq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_K6Cl8dM5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_LIyHp73F_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_Q066o47G_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_av5xPfmI_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_4TjFnNKG;
  SignalEnvelope sEnv_o4iLABqI;
  SignalEnvelope sEnv_neiSfUkE;
  SignalEnvelope sEnv_pm5YmjXV;
  SignalEnvelope sEnv_1Qu7FPO6;
  SignalLine sLine_sfc8ceWN;
  SignalLine sLine_LYudsyTa;
  ControlBinop cBinop_mQpFYpPQ;
  ControlBinop cBinop_iePXs7ST;
  ControlBinop cBinop_qietrNeU;
  ControlBinop cBinop_ZLjKZKJI;
  ControlBinop cBinop_fRuXJiqG;
  ControlBinop cBinop_9B9V5TpW;
  ControlBinop cBinop_Fevrto5j;
  ControlBinop cBinop_JOumrFhE;
  ControlVar cVar_nKJ7Qc8o;
  ControlVar cVar_qnRWmulD;
  SignalVarf sVarf_RXzGSyHe;
  ControlVar cVar_fa1OhshP;
  ControlIf cIf_oQJfRBRd;
  ControlIf cIf_Ky3QqWqh;
  ControlBinop cBinop_SWja4r7G;
  ControlBinop cBinop_oCCTvbyz;
  ControlBinop cBinop_Ov6Hs5mB;
  ControlBinop cBinop_DtwdFIjZ;
  ControlBinop cBinop_RHCeAfcI;
  SignalVarf sVarf_iRxHiwLg;
  ControlVar cVar_wTd7zwY0;
  ControlIf cIf_AGATjo6q;
  ControlIf cIf_Cs3tmMiG;
  ControlBinop cBinop_aPtwgfDv;
  ControlBinop cBinop_7CqoGX8H;
  ControlBinop cBinop_VFgZglKk;
  ControlBinop cBinop_bvfFQz3I;
  ControlBinop cBinop_1VR10VXc;
  ControlVar cVar_8WN0aAH5;
  ControlBinop cBinop_iNp1IwRl;
  SignalVarf sVarf_KeILxjtM;
  SignalVarf sVarf_n8qmAgFo;
  SignalVarf sVarf_0VcOb5aJ;
  SignalVarf sVarf_BKFPFmYR;
  ControlBinop cBinop_B9f1D9GQ;
};

#endif // _HEAVY_CONTEXT_WSTD_SMTHR_HPP_
